package springbatchdemo;

import org.springframework.core.env.Environment;
import org.springframework.stereotype.Component;
import java.io.*;
import java.util.Properties;

/**
 * Job-Properties.
 * Bitte Folgendes nicht verwechseln:
 *   - JobParameters:
 *       Von Spring Batch zur Laufzeit weitergereichte zu einer Job-Execution gehoerende Parameter
 *       (beispielsweise fuer die jeweilige Job-Ausfuehrung zu verwendende Datumsbereiche).
 *   - Job-Properties:
 *       Nicht von Spring Batch benoetigte, sondern fuer die Jobs benoetigte Properties (z.B. E-Mail-Texte).
 *   - Application-Properties:
 *       Von Spring beim Start der Anwendung zur Initialisierung der Anwendung und zum Aufbau des
 *       Spring-ApplicationContext verwendete Properties (z.B. DB-URL).
 * Diese Klasse sammelt Job-Properties. Dabei wird zwischen zwei Faellen unterschieden:
 *   - Der Kommandozeilenparameter "--spring.config.location" ist nicht gesetzt:
 *       Das ist ueblicherweise in JUnit-Modultests der Fall.
 *       Dann werden die Job-Properties-Dateien im "src/test/resources"-Verzeichnis gelesen.
 *       Dies testet der JUnit-Modultest "JobPropertiesTest".
 *   - Der Kommandozeilenparameter "--spring.config.location" ist gesetzt:
 *       Das ist in der Regel der Fall, wenn die Anwendung auerhalb der Modultests betrieben wird,
 *       beispielsweise in einem Batch-Webserver.
 *       In diesem Fall werden die Job-Properties-Dateien in demselben Verzeichnis gelesen, in dem die per
 *       "--spring.config.location" angegebene Application-Properties-Datei liegt.
 */
@Component
public class JobProperties
{
   /**
    * Lies Job-Properties aus Properties-Dateien.
    * @param  env     Spring-Environment (z.B. injiziert per: @Autowired Environment).
    * @param  prefix  "null" falls keine Filterung, sonst nur Properties-Dateien deren Name mit diesem Prefix beginnen.
    * @param  postfix "null" falls keine Filterung, sonst nur Properties-Dateien deren Name mit diesem Postfix endet (vor der Dateiendung).
    * @param  exclude "null" falls keine Filterung, sonst nur Properties-Dateien deren Name verschieden zu diesem Namen ist.
    * @return Job-Properties
    */
   public static Properties getJobProperties( Environment env, final String prefix, final String postfix, final String exclude ) throws IOException
   {
      File parent = null;
      String springConfigLocation = env.getProperty( "spring.config.location" );
      if( springConfigLocation != null && !springConfigLocation.isEmpty() ) {
         parent = (new File( springConfigLocation )).getAbsoluteFile().getParentFile();
      } else {
         parent = (new File( "src/test/resources" )).getAbsoluteFile();
      }
      return getProperties( parent, prefix, postfix, exclude );
   }

   private static Properties getProperties( File parent, final String prefix, final String postfix, final String exclude ) throws IOException
   {
      FilenameFilter filenameFilter = new FilenameFilter() {
         @Override
         public boolean accept( File dir, String name ) {
            if( (exclude != null && name.equals( exclude )) ||
                (!name.endsWith( ".properties" )) ||
                (postfix != null && !name.endsWith( postfix + ".properties" )) ||
                (prefix != null && !name.startsWith( prefix )) ) {
               return false;
            }
            return true;
         }
      };
      Properties props = new Properties();
      File[] files = parent.listFiles( filenameFilter );
      for( File f : files ) {
         FileInputStream is = new FileInputStream( f );
         try {
            Properties p = new Properties();
            p.load( is );
            props.putAll( p );
         } finally {
            is.close();
         }
      }
      return props;
   }
}
