package springbatchdemo;

import org.springframework.batch.core.*;
import org.springframework.batch.core.configuration.annotation.*;
import org.springframework.batch.core.launch.support.RunIdIncrementer;
import org.springframework.batch.core.scope.context.ChunkContext;
import org.springframework.batch.core.step.tasklet.Tasklet;
import org.springframework.batch.repeat.RepeatStatus;
import org.springframework.beans.factory.annotation.*;
import org.springframework.context.ConfigurableApplicationContext;
import org.springframework.context.annotation.*;
import org.springframework.core.env.*;
import java.io.*;
import java.util.*;

/**
 * Job zur Ausgabe auf der Konsole von:
 *   - Spring-Environment inklusive Application-Properties,
 *   - alle JobProperties (aus eventuell mehreren Properties-Dateien)
 *   - und gezielt eine einzelne JobProperty (falls deren Key angegeben wird).
 * Um eine einzelne JobProperty auszugeben, muss deren Key als JobParameter mit dem Key "PropKey" angegeben werden.
 * Angenommen der Job ist in einem Batch-Webserver mit REST-Schnittstelle integriert und
 * die URL zum Starten des Jobs entspricht der im Folgenden gezeigten (bitte anpassen),
 * dann kann dies bei laufendem Batch-Webserver folgendermassen per REST getestet werden, z.B. so:
 * Unter Linux:
 *   curl --request POST localhost:8080/jobs/meinPrintEnvJob?Zeit=$(date -Iseconds),PropKey=Test-Prop-Key
 * Unter Windows:
 *   curl --request POST localhost:8080/jobs/meinPrintEnvJob?Zeit=%time%,PropKey=Test-Prop-Key
 */
@Configuration
@EnableBatchProcessing
public class PrintEnvJobConfiguration
{
   public static final String JOB_NAME = "meinPrintEnvJob";
   @Autowired private ConfigurableApplicationContext ctx;
   @Autowired private JobBuilderFactory  jobBuilderFactory;
   @Autowired private StepBuilderFactory stepBuilderFactory;

   @Bean
   public Job meinPrintEnvJob() throws Exception
   {
      return jobBuilderFactory.get( JOB_NAME ).incrementer( new RunIdIncrementer() )
              .start( meinPrintEnvStep() )
              .build();
   }

   @Bean
   public Step meinPrintEnvStep()
   {
      return stepBuilderFactory.get( "meinPrintEnvStep" ).tasklet( new Tasklet()
      {
         @Override
         public RepeatStatus execute( StepContribution sc, ChunkContext cc ) throws IOException
         {
            ConfigurableEnvironment env = ctx.getEnvironment();
            printSpringEnvironment( env, null );

            File parent = null;
            String name = null;
            String springConfigLocation = env.getProperty( "spring.config.location" );
            if( springConfigLocation != null ) {
               File f = (new File( springConfigLocation )).getAbsoluteFile();
               parent = f.getParentFile();
               name   = f.getName();
            }
            System.out.println( "" );
            System.out.println( "spring.config.location-Parameter: " + springConfigLocation );
            System.out.println( "spring.config.location-ParentDir: " + parent );
            System.out.println( "\n-----------------------------------------\n" );

            System.out.println( "JobProperties (eventuell aus mehreren Properties-Dateien):" );
            Properties jobProps = JobProperties.getJobProperties( env, null, null, name );
            TreeMap<Object,Object> mp = new TreeMap<Object,Object>( jobProps );
            for( Map.Entry<Object,Object> e : mp.entrySet() ) {
               System.out.println( "    " + e.getKey() + " = " + e.getValue() );
            }
            System.out.println( "\n-----------------------------------------" );

            String propKey = cc.getStepContext().getStepExecution().getJobParameters().getString( "PropKey" );
            if( propKey != null ) {
               System.out.println( "Job-Property: " + propKey + " = " + jobProps.get( propKey ) );
               System.out.println( "-----------------------------------------" );
            }
            System.out.println( "\n" );

            return RepeatStatus.FINISHED;
         }
      } ).build();
   }

   /**
    * Anzeige von Spring-Environment-Properties (nur waehrend Entwicklung sinnvoll).
    * @param env ConfigurableEnvironment, z.B. mit: ConfigurableApplicationContext.getEnvironment()
    *                                          oder ConfigFileApplicationListener.postProcessEnvironment()
    *                                          oder ApplicationEnvironmentPreparedEvent.getEnvironment()
    * @param onlyPropSrcName Falls null: Anzeige aller Properties; sonst: nur Props aus PropertySource mit diesem Namen
    */
   public static void printSpringEnvironment( ConfigurableEnvironment env, String onlyPropSrcName )
   {
      for( Iterator<org.springframework.core.env.PropertySource<?>> itr = env.getPropertySources().iterator(); itr.hasNext(); ) {
         org.springframework.core.env.PropertySource<?> ps = itr.next();
         if( onlyPropSrcName != null && ps.getName() != null && !ps.getName().contains( onlyPropSrcName ) ) {
            continue;
         }
         System.out.println( "-----------------------------------------" );
         System.out.println( ps.getName() + " (" + ps.getClass() + ")" );
         if( ps instanceof EnumerablePropertySource ) {
            EnumerablePropertySource<?> eps = (EnumerablePropertySource<?>) ps;
            String[] pns = eps.getPropertyNames();
            Arrays.sort( pns );
            for( String pn : pns ) {
               System.out.println( "    " + pn + " = " + eps.getProperty( pn ) );
            }
         } else {
            System.out.println( "    PropertySource: " + ps.getSource().getClass() );
         }
      }
      System.out.println( "-----------------------------------------" );
   }
}
